#!/bin/sh

BASEDIR=`pwd`
. ${BASEDIR%scripts}/rrdtool.conf

TITLE="Ram usage"
BASENAME=`basename $0`
FILE=${BASENAME%.sh}
DBASE="${DBASEDIR}/${FILE}.rrd"
IMG="${IMGDIR}/${FILE}.png"
SIMG="${IMGDIR}/small-${FILE}.png"

if [ ! -f "${DBASE}" ]; then

${RRD} create ${DBASE} \
        --start N \
        --step 300 \
        DS:ram_phi:GAUGE:600:0:U \
        DS:ram_tot:GAUGE:600:0:U \
        DS:ram_wir:GAUGE:600:0:U \
        DS:ram_act:GAUGE:600:0:U \
        DS:ram_ina:GAUGE:600:0:U \
        DS:ram_cac:GAUGE:600:0:U \
        DS:ram_fre:GAUGE:600:0:U \
        DS:ram_tfr:GAUGE:600:0:U \
        DS:ram_tus:GAUGE:600:0:U \
        RRA:AVERAGE:0.5:1:288
fi

PHI=`sysctl -a | grep hw.physmem | awk '{print $2}'`
TOT=`sysctl -a | grep vm.stats.vm.v_page_count | awk '{print $2 * 4096}'`
WIR=`sysctl -a | grep vm.stats.vm.v_wire_count | awk '{print $2 * 4096}'`
ACT=`sysctl -a | grep vm.stats.vm.v_active_count | awk '{print $2 * 4096}'`
INA=`sysctl -a | grep vm.stats.vm.v_inactive_count | awk '{print $2 * 4096}'`
CAC=`sysctl -a | grep vm.stats.vm.v_cache_count | awk '{print $2 * 4096}'`
FRE=`sysctl -a | grep vm.stats.vm.v_free_count | awk '{print $2 * 4096}'`

let TFR = ${FRE} + ${CAC} + ${INA}
let TUS = ${PHI} - ${TFR}


${RRD} update ${DBASE} N:${PHI}:${TOT}:${WIR}:${ACT}:${INA}:${CAC}:${FRE}:${TFR}:${TUS}

${RRD} graph ${IMG} \
        --imgformat=PNG \
        --title="${TITLE}" \
        --font DEFAULT:7 \
        --height=100 \
        --width=400 \
        --lower-limit=0 \
        --base=1024 \
        DEF:phi="${DBASE}":ram_phi:AVERAGE \
        DEF:tot="${DBASE}":ram_tot:AVERAGE \
        DEF:wir="${DBASE}":ram_wir:AVERAGE \
        DEF:act="${DBASE}":ram_act:AVERAGE \
        DEF:ina="${DBASE}":ram_ina:AVERAGE \
        DEF:cac="${DBASE}":ram_cac:AVERAGE \
        DEF:fre="${DBASE}":ram_fre:AVERAGE \
        DEF:tfr="${DBASE}":ram_tfr:AVERAGE \
        DEF:tus="${DBASE}":ram_tus:AVERAGE \
        GPRINT:phi:LAST:"System phisical memory\:%4.0lf %s"\
        GPRINT:tot:LAST:"Total real memory managed\:%4.0lf %s\n"\
        COMMENT:" \n"\
        AREA:tfr#00CF00:"Total free\t\t\t\t\t" GPRINT:tfr:LAST:"Last\:%4.0lf %s\n"\
        AREA:tus#ff0000:"Total used\t\t\t\t\t" GPRINT:tus:LAST:"Last\:%4.0lf %s\n"\
        COMMENT:" \n"\
        LINE1:wir#002A97:"Wired\: disabled for paging out\t\t" GPRINT:wir:LAST:"Last\:%4.0lf %s\n"\
        LINE1:act#191970:"Active\: recently referenced\t\t" GPRINT:act:LAST:"Last\:%4.0lf %s\n"\
        COMMENT:" \n"\
        LINE1:ina#FFE4B5:"Inactive\: recently not referenced\t" GPRINT:ina:LAST:"Last\:%4.0lf %s\n"\
        LINE1:cac#FFD700:"Cached\: almost usable for allocation\t" GPRINT:cac:LAST:"Last\:%4.0lf %s\n"\
        LINE1:fre#FFFF00:"Free\: fully available for allocation\t" GPRINT:fre:LAST:"Last\:%4.0lf %s\n"\

${RRD} graph ${SIMG} \
        --imgformat=PNG \
        --title="${TITLE}" \
        --font DEFAULT:5 \
        --height=50 \
        --width=200 \
        --lower-limit=0 \
        --no-legend \
        --base=1024 \
        DEF:phi="${DBASE}":ram_phi:AVERAGE \
        DEF:tot="${DBASE}":ram_tot:AVERAGE \
        DEF:wir="${DBASE}":ram_wir:AVERAGE \
        DEF:act="${DBASE}":ram_act:AVERAGE \
        DEF:ina="${DBASE}":ram_ina:AVERAGE \
        DEF:cac="${DBASE}":ram_cac:AVERAGE \
        DEF:fre="${DBASE}":ram_fre:AVERAGE \
        DEF:tfr="${DBASE}":ram_tfr:AVERAGE \
        DEF:tus="${DBASE}":ram_tus:AVERAGE \
        AREA:tfr#00CF00:\
        AREA:tus#ff0000:\

        # Optionals
        # HRULE:150000#FF0000:"MAX possible speed"
        # CDEF:byte_in=in,8,* \
        # CDEF:byte_out=out,8,* \
        # LINE1:phi#002A97:"Available Ram\t\t\t\t" GPRINT:phi:LAST:"Last\:%4.0lf %s\n"\
        # LINE1:tot#191970:"Total real memory managed\t\t" GPRINT:tot:LAST:"Last\:%4.0lf %s\n"\

